/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * \package    RobotAPI::gui-plugins::LaserScannerPluginWidgetController
 * \author     Fabian Paus ( fabian dot paus at kit dot edu )
 * \date       2017
 * \copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "LaserScannerPluginWidgetController.h"

#include <QGraphicsView>
#include <QGraphicsLineItem>

#include <string>

using namespace armarx;

LaserScannerPluginWidgetController::LaserScannerPluginWidgetController()
{
    widget.setupUi(getWidget());

    widget.graphicsView->setScene(&scene);
}


LaserScannerPluginWidgetController::~LaserScannerPluginWidgetController()
{

}


void LaserScannerPluginWidgetController::loadSettings(QSettings* settings)
{

}

void LaserScannerPluginWidgetController::saveSettings(QSettings* settings)
{

}


void LaserScannerPluginWidgetController::onInitComponent()
{
    usingProxy(laserScannerUnitName);

    connect(this, SIGNAL(newSensorValuesReported()), this, SLOT(onNewSensorValuesReported()), Qt::QueuedConnection);
}


void LaserScannerPluginWidgetController::onConnectComponent()
{
    laserScannerUnit = getProxy<LaserScannerUnitInterfacePrx>(laserScannerUnitName);
    std::string topicName = laserScannerUnit->getReportTopicName();
    usingTopic(topicName);
}

QPointer<QDialog> LaserScannerPluginWidgetController::getConfigDialog(QWidget* parent)
{
    if (!dialog)
    {
        dialog = new SimpleConfigDialog(parent);
        dialog->addProxyFinder<LaserScannerUnitInterfacePrx>({"LaserScannerUnit", "", "*"});
    }
    return qobject_cast<SimpleConfigDialog*>(dialog);
}

void LaserScannerPluginWidgetController::configured()
{
    if (dialog)
    {
        laserScannerUnitName = dialog->getProxyName("LaserScannerUnit");
    }
}

void LaserScannerPluginWidgetController::reportSensorValues(const std::string& device, const std::string& name, const LaserScan& newScan, const TimestampBasePtr& timestamp, const Ice::Current& c)
{
    {
        boost::mutex::scoped_lock lock(scanMutex);

        LaserScan& scan = scans[device];
        // TODO: Do some filtering? aggregation?
        scan = newScan;
    }

    emit newSensorValuesReported();
}

void LaserScannerPluginWidgetController::onNewSensorValuesReported()
{
    // TODO: Draw something on the canvas

    QComboBox* deviceBox = widget.deviceComboBox;

    boost::mutex::scoped_lock lock(scanMutex);
    for (auto & pair : scans)
    {
        QString deviceName(QString::fromStdString(pair.first.c_str()));
        if (deviceBox->findText(deviceName) < 0)
        {
            deviceBox->addItem(deviceName);
        }
    }

    std::string deviceName(deviceBox->currentText().toUtf8().data());

    QGraphicsView* view = widget.graphicsView;


    scene.clear();
    int outerR = std::min(view->width() / 2, view->height() / 2);
    scene.addEllipse(-outerR, -outerR, 2 * outerR, 2 * outerR, QPen(QColor(255, 255, 255)));
    int r = outerR - 10;
    scene.addEllipse(-r, -r, 2 * r, 2 * r, QPen(QColor(200, 200, 200)));
    QColor stepColor(QColor::fromRgb(100, 100, 255));
    QPen stepPen(stepColor);
    QBrush stepBrush(stepColor);
    auto line = [&](float angle, float d)
    {
        float di = d * r;
        QGraphicsEllipseItem* item = scene.addEllipse(-di, -di, 2 * di, 2 * di, stepPen, stepBrush);
        // Angles for Qt ellipse are in 16th of degree (who thought that would be a great idea?)
        item->setStartAngle(std::round(16.0f * angle * 180.0 / M_PI));
        item->setSpanAngle(std::round(0.25f * 16.0f));
        //scene.addLine(0, 0, std::sin(angle) * d * r, std::cos(angle) * d * r);
    };

    LaserScan& scan = scans[deviceName];
    for (LaserScanStep & step : scan)
    {
        line(step.angle, step.distance / 10000.0f);
    }

    view->fitInView(scene.itemsBoundingRect(), Qt::KeepAspectRatio);
}
